#! /bin/sh
#
# patchymf.sh release 1.60
# by ITO Takayuki <yuki@fsinet.or.jp>
# http://hp.vector.co.jp/authors/VA001240/
#
# This script applies patch to BeOS R4.5.x/R5 YMF724 driver
# for use with some other Yamaha PCI audio chipsets
# and installs it to appropriate place.
#
# 1.00	first release
# 1.10	R4.5 experimental driver supported
# 1.20	R4.5.2 driver supported
# 1.30	YMF754 supported (untested)
# 1.40	Independent of driver version
#	GUI
# 1.50	automated; YMF740B supported
# 1.60	R5 supported for YMF754
#
# Get YMF724F specifications at:
# http://www.yamahayst.com/pcaudio/products.htm
#
# PCI vendor ID: 1073h (Yamaha)
# PCI device IDs:
# YMF724D (DS-1)  0004h
# YMF724F (DS-1)  000Dh
# YMF740B (DS-1L) 000Ah
# YMF740C (DS-1L) 000Ch
# YMF744B (DS-1S) 0010h
# YMF754  (DS-1E) 0012h

SRCDIR=/system/add-ons/kernel/drivers
DSTDIR=$HOME/config/add-ons/kernel/drivers
FILE=ymf724

cd "`dirname \"$0\"`"		# for being called from Tracker

case `uname -r` in
4.5)
	;;
5.0)
	;;
*)
	alert --stop "For R4.5.x/R5 only."
	exit 1
	;;
esac

alert --warning "This script will apply patch to the ymf724 driver for use with another Yamaha chipset." OK Quit > /dev/null
if [ $? -eq 1 ]
then
	alert --stop "Aborted."
	exit 1
fi

if [ ! -f ./bsubst ]
then
	alert --stop "bsubst is missing."
	exit 1
fi

# check original driver

if [ ! -f $SRCDIR/bin/$FILE ]
then
	alert --stop "Original driver $SRCDIR/bin/$FILE not present."
	exit 1
fi

# chipset detection

TMPFILE=/tmp/poke.log

poke > $TMPFILE <<END
pci
quit
END

# XXX: assumes all Yamaha devices are audio devices and looks at the first device only
ID=`grep 'vendor 1073' $TMPFILE | head -1 | cut -d ' ' -f 10`
rm $TMPFILE
if [ "x$ID" = x ]
then
	alert --stop "No Yamaha audio chipset detected."
	exit 1
fi

IDH=00
case $ID in
0004)	CHIP=ymf724d
	IDL=04;;
000d)	CHIP=ymf724f
	IDL=0d;;
000a)	CHIP=ymf740b
	IDL=0a;;
000c)	CHIP=ymf740c
	IDL=0c;;
0010)	CHIP=ymf744b
	IDL=10;;
0012)	CHIP=ymf754
	IDL=12;;
*)	CHIP=yamaha_unknown;;
esac
if [ $CHIP = yamaha_unknown ]
then
	alert --info "Unknown Yamaha audio chipset detected. Device ID is $ID."
else
	alert --info "Yamaha audio chipset $CHIP detected. Device ID is $ID."
fi

case `uname -r` in
4.5)
	SRC="73 10 04 00"
	DST="73 10 $IDL $IDH"
	;;
5.0)
	SRC="73 10 00 00 10 00 00 00"
	DST="73 10 00 00 $IDL $IDH 00 00"
	;;
esac

# apply patch to driver

if [ -f $DSTDIR/bin/$CHIP ]
then
	alert --stop "Patch is already applied."
	exit 1
fi

alert --info "Processing $SRCDIR/bin/$FILE."
./bsubst "$SRC" "$DST" < $SRCDIR/bin/$FILE > $DSTDIR/bin/$CHIP
cmp -s $SRCDIR/bin/$FILE $DSTDIR/bin/$CHIP
if [ $? -eq 0 ]
then
	alert --stop "Patch is not applicable. Nothing altered."
	rm $DSTDIR/bin/$CHIP
	exit 1
fi

mkdir -p $DSTDIR/dev/audio/old
[ -h $DSTDIR/dev/audio/old/$CHIP ] || \
ln -s ../../../bin/$CHIP $DSTDIR/dev/audio/old/$CHIP

alert --info "Done."
exit 0
